import setuptools

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setuptools.setup(
    name="python-core", # your package name
    version="0.0.3", # your package version
    author="alexzander", # package author
    author_email="just.python.mail.test@gmail.com", # package author email
    description="", # package description
    long_description=long_description, # package long description
    long_description_content_type="text/markdown",
    url="",
    include_package_data=True,
    install_requires=[ # all installed 3rd party modules
        "aiohttp",
		"altgraph",
		"appdirs",
		"argon2-cffi",
		"args",
		"asgiref",
		"astroid",
		"async-timeout",
		"attrs",
		"autopep8",
		"backcall",
		"beautifulsoup4",
		"bitly-api",
		"bleach",
		"bs4",
		"cachetools",
		"cairocffi",
		"CairoSVG",
		"certifi",
		"cffi",
		"chardet",
		"click",
		"clint",
		"colorama",
		"colored-traceback",
		"comtypes",
		"cssselect",
		"cssselect2",
		"datedelta",
		"decorator",
		"defusedxml",
		"Django",
		"docutils",
		"entrypoints",
		"fake-useragent",
		"ffmpeg-python",
		"future",
		"geojson",
		"googletrans",
		"gTTS",
		"h11",
		"h2",
		"hpack",
		"hstspreload",
		"httpcore",
		"httpx",
		"hyperframe",
		"idna",
		"imageio",
		"imageio-ffmpeg",
		"importlib-metadata",
		"InstagramAPI",
		"ipinfo",
		"ipykernel",
		"ipython",
		"ipython-genutils",
		"ipywidgets",
		"isort",
		"jedi",
		"Jinja2",
		"jsonschema",
		"jupyter-client",
		"jupyter-console",
		"jupyter-core",
		"keyboard",
		"keyring",
		"Kivy",
		"kivy-deps.angle",
		"kivy-deps.glew",
		"kivy-deps.gstreamer",
		"kivy-deps.sdl2",
		"Kivy-examples",
		"Kivy-Garden",
		"lazy-object-proxy",
		"lxml",
		"MarkupSafe",
		"mccabe",
		"mistune",
		"MouseInfo",
		"moviepy",
		"multidict",
		"mysqlclient",
		"nbconvert",
		"nbformat",
		"notebook",
		"numpy",
		"opencv-python",
		"packaging",
		"pandocfilters",
		"parse",
		"parso",
		"pdfminer.six",
		"pdfplumber",
		"pefile",
		"pickleshare",
		"Pillow",
		"pkginfo",
		"playsound",
		"proglog",
		"prometheus-client",
		"prompt-toolkit",
		"psutil",
		"PyAudio",
		"PyAutoGUI",
		"pycodestyle",
		"pycparser",
		"pycryptodome",
		"pydub",
		"pyee",
		"pygal",
		"pygame",
		"PyGetWindow",
		"Pygments",
		"pygubu",
		"pygubu-designer",
		"pyinstaller",
		"pyinstaller-hooks-contrib",
		"pylint",
		"PyMsgBox",
		"pynput",
		"pyowm",
		"pyparsing",
		"PyPDF2",
		"pyperclip",
		"pypiwin32",
		"pyppeteer",
		"PyQt5",
		"PyQt5-sip",
		"pyquery",
		"PyRect",
		"pyrsistent",
		"PyScreeze",
		"PySocks",
		"pytesseract",
		"python-core",
		"python-dateutil",
		"pyttsx3",
		"PyTweening",
		"pytz",
		"pywin32",
		"pywin32-ctypes",
		"pywinpty",
		"pyzbar",
		"pyzmq",
		"qtconsole",
		"QtPy",
		"readme-renderer",
		"requests",
		"requests-html",
		"requests-toolbelt",
		"rfc3986",
		"scipy",
		"selenium",
		"Send2Trash",
		"six",
		"sniffio",
		"sortedcontainers",
		"SoundCard",
		"sounddevice",
		"soupsieve",
		"SpeechRecognition",
		"spotipy",
		"sqlparse",
		"terminado",
		"testpath",
		"tinycss2",
		"toml",
		"tornado",
		"tqdm",
		"traitlets",
		"twine",
		"typed-ast",
		"typing-extensions",
		"urllib3",
		"vboxapi",
		"w3lib",
		"Wand",
		"wcwidth",
		"webencodings",
		"websockets",
		"widgetsnbextension",
		"win10toast",
		"windows-curses",
		"wrapt",
		"yarl",
		"zipp"
    ],
    platforms="any",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)